@@ -3,14 +3,14 @@
 #pragma once
 
 #include <spicy/compiler/detail/codegen/productions/boolean.h>
-#include <spicy/compiler/detail/codegen/productions/byte-block.h>
 #include <spicy/compiler/detail/codegen/productions/counter.h>
 #include <spicy/compiler/detail/codegen/productions/ctor.h>
+#include <spicy/compiler/detail/codegen/productions/deferred.h>
 #include <spicy/compiler/detail/codegen/productions/enclosure.h>
 #include <spicy/compiler/detail/codegen/productions/epsilon.h>
 #include <spicy/compiler/detail/codegen/productions/for-each.h>
 #include <spicy/compiler/detail/codegen/productions/look-ahead.h>
-#include <spicy/compiler/detail/codegen/productions/resolved.h>
+#include <spicy/compiler/detail/codegen/productions/reference.h>
 #include <spicy/compiler/detail/codegen/productions/sequence.h>
 #include <spicy/compiler/detail/codegen/productions/skip.h>
 #include <spicy/compiler/detail/codegen/productions/switch.h>
