@@ -20,9 +20,10 @@ def configure(verbosity=0, rich_logging=False, stream=None):
 
     if rich_logging:
         formatter = logging.Formatter(
-            '%(asctime)s %(levelname)-8s %(message)s', '%Y-%m-%d %H:%M:%S')
+            "%(asctime)s %(levelname)-8s %(message)s", "%Y-%m-%d %H:%M:%S"
+        )
     else:
-        formatter = logging.Formatter('%(levelname)s: %(message)s')
+        formatter = logging.Formatter("%(levelname)s: %(message)s")
 
     handler = logging.StreamHandler(stream=stream)
     handler.setFormatter(formatter)
