@@ -34,19 +34,7 @@ public:
      */
     factory_callback Factory() const { return factory; }
 
-    /**
-     * Returns true if the analyzer is currently enabled and hence
-     * available for use.
-     */
-    bool Enabled() const { return enabled; }
-
-    /**
-     * Enables or disables this analyzer.
-     *
-     * @param arg_enabled True to enabled, false to disable.
-     *
-     */
-    void SetEnabled(bool arg_enabled);
+    void SetEnabled(bool arg_enabled) override;
 
 protected:
     /**
@@ -56,7 +44,6 @@ protected:
 
 private:
     factory_callback factory; // The analyzer's factory callback.
-    bool enabled = true;      // True if the analyzer is enabled.
 };
 
 } // namespace zeek::packet_analysis
