@@ -5,6 +5,7 @@
 #pragma once
 
 #include "zeek/Event.h"
+#include "zeek/script_opt/Expr.h"
 #include "zeek/script_opt/ProfileFunc.h"
 #include "zeek/script_opt/UseDefs.h"
 #include "zeek/script_opt/ZAM/ZBody.h"
@@ -84,6 +85,13 @@ public:
     void Dump();
 
 private:
+    friend class SimpleZBI;
+    friend class CondZBI;
+    friend class OptAssignZBI;
+    friend class SortZBI;
+    friend class CatZBI;
+    friend class MultiZBI;
+
     void Init();
     void InitGlobals();
     void InitArgs();
@@ -129,8 +137,6 @@ private:
     const ZAMStmt CompileExpr(const ExprStmt* es);
     const ZAMStmt CompileIf(const IfStmt* is);
     const ZAMStmt CompileSwitch(const SwitchStmt* sw);
-    const ZAMStmt CompileAdd(const AddStmt* as);
-    const ZAMStmt CompileDel(const DelStmt* ds);
     const ZAMStmt CompileWhile(const WhileStmt* ws);
     const ZAMStmt CompileFor(const ForStmt* f);
     const ZAMStmt CompileReturn(const ReturnStmt* r);
@@ -178,9 +184,13 @@ private:
 
     const ZAMStmt CompileIncrExpr(const IncrExpr* e);
     const ZAMStmt CompileAppendToExpr(const AppendToExpr* e);
+    const ZAMStmt CompileAdd(const AggrAddExpr* e);
+    const ZAMStmt CompileDel(const AggrDelExpr* e);
     const ZAMStmt CompileAddToExpr(const AddToExpr* e);
     const ZAMStmt CompileRemoveFromExpr(const RemoveFromExpr* e);
     const ZAMStmt CompileAssignExpr(const AssignExpr* e);
+    const ZAMStmt CompileRecFieldUpdates(const RecordFieldUpdatesExpr* e);
+    const ZAMStmt CompileZAMBuiltin(const NameExpr* lhs, const ScriptOptBuiltinExpr* zbi);
     const ZAMStmt CompileAssignToIndex(const NameExpr* lhs, const IndexExpr* rhs);
     const ZAMStmt CompileFieldLHSAssignExpr(const FieldLHSAssignExpr* e);
     const ZAMStmt CompileScheduleExpr(const ScheduleExpr* e);
@@ -228,13 +238,16 @@ private:
     const ZAMStmt Call(const ExprStmt* e);
     const ZAMStmt AssignToCall(const ExprStmt* e);
     const ZAMStmt DoCall(const CallExpr* c, const NameExpr* n);
+    bool CheckForBuiltIn(const ExprPtr& e, CallExprPtr c);
 
     const ZAMStmt AssignVecElems(const Expr* e);
     const ZAMStmt AssignTableElem(const Expr* e);
 
     const ZAMStmt ConstructTable(const NameExpr* n, const Expr* e);
     const ZAMStmt ConstructSet(const NameExpr* n, const Expr* e);
-    const ZAMStmt ConstructRecord(const NameExpr* n, const Expr* e);
+    const ZAMStmt ConstructRecord(const NameExpr* n, const Expr* e) { return ConstructRecord(n, e, false); }
+    const ZAMStmt ConstructRecordFromRecord(const NameExpr* n, const Expr* e) { return ConstructRecord(n, e, true); }
+    const ZAMStmt ConstructRecord(const NameExpr* n, const Expr* e, bool is_from_rec);
     const ZAMStmt ConstructVector(const NameExpr* n, const Expr* e);
 
     const ZAMStmt ArithCoerce(const NameExpr* n, const Expr* e);
@@ -244,17 +257,8 @@ private:
 
     const ZAMStmt Is(const NameExpr* n, const Expr* e);
 
-#include "zeek/script_opt/ZAM/BuiltIn.h"
 #include "zeek/script_opt/ZAM/Inst-Gen.h"
 
-    // A bit weird, but handy for switch statements used in built-in
-    // operations: returns a bit mask of which of the arguments in the
-    // given list correspond to constants, with the high-ordered bit
-    // being the first argument (argument "0" in the list) and the
-    // low-ordered bit being the last.  Second parameter is the number
-    // of arguments that should be present.
-    zeek_uint_t ConstArgsMask(const ExprPList& args, int nargs) const;
-
     int ConvertToInt(const Expr* e) {
         if ( e->Tag() == EXPR_NAME )
             return FrameSlot(e->AsNameExpr()->Id());
@@ -320,7 +324,7 @@ private:
 
     // Returns a handle to state associated with building
     // up a list of values.
-    OpaqueVals* BuildVals(const ListExprPtr&);
+    std::unique_ptr<OpaqueVals> BuildVals(const ListExprPtr&);
 
     // "stride" is how many slots each element of l will consume.
     ZInstAux* InternalBuildVals(const ListExpr* l, int stride = 1);
@@ -361,7 +365,7 @@ private:
     int FrameSlot(const ID* id);
     int FrameSlotIfName(const Expr* e) {
         auto n = e->Tag() == EXPR_NAME ? e->AsNameExpr() : nullptr;
-        return n ? FrameSlot(n->Id()) : 0;
+        return n ? FrameSlot(n->Id()) : -1;
     }
 
     int FrameSlot(const NameExpr* id) { return FrameSlot(id->AsNameExpr()->Id()); }
