@@ -60,16 +60,22 @@ public:
     ZInst(ZOp _op, ZAMOpType _op_type) {
         op = _op;
         op_type = _op_type;
+        ASSERT(ZAM::curr_loc);
+        loc = ZAM::curr_loc;
     }
 
     // Create a stub instruction that will be populated later.
-    ZInst() = default;
+    ZInst() {
+        ASSERT(ZAM::curr_loc);
+        loc = ZAM::curr_loc;
+    }
 
     virtual ~ZInst() = default;
 
-    // Methods for printing out the instruction for debugging/maintenance.
-    void Dump(zeek_uint_t inst_num, const FrameReMap* mappings) const;
-    void Dump(const std::string& id1, const std::string& id2, const std::string& id3, const std::string& id4) const;
+    // Methods for printing out the instruction for debugging/profiling.
+    void Dump(FILE* f, zeek_uint_t inst_num, const FrameReMap* mappings, const std::string& prefix) const;
+    void Dump(FILE* f, const std::string& prefix, const std::string& id1, const std::string& id2,
+              const std::string& id3, const std::string& id4) const;
 
     // Returns the name to use in identifying one of the slots/integer
     // values (designated by "n").  "inst_num" identifies the instruction
@@ -92,6 +98,14 @@ public:
     // a loop iteration, false otherwise.
     bool IsLoopIterationAdvancement() const;
 
+    // True if the given instruction assigns to the frame location
+    // given by slot 1 (v1).
+    bool AssignsToSlot1() const;
+
+    // True if the given instruction assigns to the frame location
+    // corresponding to the given slot.
+    bool AssignsToSlot(int slot) const;
+
     // Returns a string describing the constant.
     std::string ConstDump() const;
 
@@ -112,11 +126,7 @@ public:
 
     // Type, usually for interpreting the constant.
     TypePtr t = nullptr;
-    TypePtr t2 = nullptr;                  // just a few ops need two types
-    const Expr* e = nullptr;               // only needed for "when" expressions
-    Func* func = nullptr;                  // used for calls
-    EventHandler* event_handler = nullptr; // used for referring to events
-    AttributesPtr attrs = nullptr;         // used for things like constructors
+    TypePtr t2 = nullptr; // just a few ops need two types
 
     // Auxiliary information.  We could in principle use this to
     // consolidate a bunch of the above, though at the cost of
@@ -124,12 +134,9 @@ public:
     // which is why we bundle these separately.
     ZInstAux* aux = nullptr;
 
-    // Location associated with this instruction, for error reporting.
-    std::shared_ptr<Location> loc;
-
-    // Interpreter call expression associated with this instruction,
-    // for error reporting and stack backtraces.
-    CallExprPtr call_expr = nullptr;
+    // Location associated with this instruction, for error reporting
+    // and profiling.
+    std::shared_ptr<ZAMLocInfo> loc;
 
     // Whether v1 represents a frame slot type for which we
     // explicitly manage the memory.
@@ -193,7 +200,7 @@ public:
     ZInstI() {}
 
     // If "remappings" is non-nil, then it is used instead of frame_ids.
-    void Dump(const FrameMap* frame_ids, const FrameReMap* remappings) const;
+    void Dump(FILE* f, const FrameMap* frame_ids, const FrameReMap* remappings) const;
 
     // Note that this is *not* an override of the base class's VName
     // but instead a method with similar functionality but somewhat
@@ -219,14 +226,6 @@ public:
     // should not be pruned even if it has a dead assignment.
     bool HasSideEffects() const;
 
-    // True if the given instruction assigns to the frame location
-    // given by slot 1 (v1).
-    bool AssignsToSlot1() const;
-
-    // True if the given instruction assigns to the frame location
-    // corresponding to the given slot.
-    bool AssignsToSlot(int slot) const;
-
     // True if the given instruction uses the value in the given frame
     // slot. (Assigning to the slot does not constitute using the value.)
     bool UsesSlot(int slot) const;
@@ -294,9 +293,6 @@ public:
     // a branch target).
     int num_labels = 0;
 
-    // Used for debugging.  Transformed into the ZInst "loc" field.
-    StmtPtr stmt = curr_stmt;
-
 private:
     // Initialize 'c' from the given ConstExpr.
     void InitConst(const ConstExpr* ce);
@@ -467,25 +463,55 @@ public:
     // Used for accessing function names.
     IDPtr id_val = nullptr;
 
+    // Interpreter call expression associated with this instruction,
+    // for error reporting and stack backtraces.
+    CallExprPtr call_expr = nullptr;
+
+    // Used for direct calls.
+    Func* func = nullptr;
+
+    // Whether we know that we're calling a BiF.
+    bool is_BiF_call = false;
+
+    // Used for referring to events.
+    EventHandler* event_handler = nullptr;
+
+    // Used for things like constructors.
+    AttributesPtr attrs = nullptr;
+
     // Whether the instruction can lead to globals/captures changing.
     // Currently only needed by the optimizer, but convenient to
     // store here.
     bool can_change_non_locals = false;
 
-    // The following is used for constructing records, to map elements in
-    // slots/constants/types to record field offsets.
+    // The following is used for constructing records or in record chain
+    // operations, to map elements in slots/constants/types to record field
+    // offsets.
     std::vector<int> map;
 
+    // The following is used when we need two maps, a LHS one (done with
+    // the above) and a RHS one.
+    std::vector<int> rhs_map;
+
+    // ... and the following when we need *three* (for constructing certain
+    // types of records). We could hack it in by adding onto "map" but
+    // this is cleaner, and we're not really concerned with the size of
+    // ZAM auxiliary information as it's not that commonly used, and doesn't
+    // grow during execution.
+    std::vector<int> lhs_map;
+
+    // For operations that need to track types corresponding to other vectors.
+    std::vector<TypePtr> types;
+
+    // For operations that mix managed and unmanaged assignments.
+    std::vector<bool> is_managed;
+
     ///// The following four apply to looping over the elements of tables.
 
     // Frame slots of iteration variables, such as "[v1, v2, v3] in aggr".
     // A negative value means "skip assignment".
     std::vector<int> loop_vars;
 
-    // Their types and whether they're managed.
-    std::vector<TypePtr> loop_var_types;
-    std::vector<bool> lvt_is_managed;
-
     // Type associated with the "value" entry, for "k, value in aggr"
     // iteration.
     TypePtr value_var_type;
